/// <reference types="node" />
import { AppStateStatus } from 'react-native';
import type { Config, JsonMap, GroupTraits, UserTraits, SegmentEvent, Store, SegmentAPISettings } from './types';
import type { Logger } from './logger';
import type { Unsubscribe } from '@reduxjs/toolkit';
import type { Persistor } from 'redux-persist';
import { Timeline } from './timeline';
import type { PlatformPlugin, Plugin } from './plugin';
import type { actions as ReduxActions } from './store';
export declare class SegmentClient {
    config: Config;
    store: Store;
    actions: typeof ReduxActions;
    persistor: Persistor;
    secondsElapsed: number;
    appState: AppStateStatus | 'unknown';
    appStateSubscription: any;
    logger: Logger;
    refreshTimeout: NodeJS.Timeout | null;
    interval: NodeJS.Timeout | null;
    unsubscribe: Unsubscribe | null;
    destroyed: boolean;
    timeline: Timeline;
    private isReady;
    private pluginsToAdd;
    get platformPlugins(): PlatformPlugin[];
    settings(): SegmentAPISettings | undefined;
    constructor({ config, logger, store, actions, persistor, }: {
        config: Config;
        logger: Logger;
        store: any;
        persistor: Persistor;
        actions: any;
    });
    platformStartup(): void;
    configure(): void;
    getSettings(): Promise<void>;
    /**
     * There is no garbage collection in JS, which means that any listeners, timeouts and subscriptions
     * would run until the application closes
     *
     * This method exists in case the user for some reason needs to recreate the class instance during runtime.
     * In this case, they should run client.cleanup() to destroy the listeners in the old client before creating a new one.
     *
     * There is a Stage 3 EMCAScript proposal to add a user-defined finalizer, which we could potentially switch to if
     * it gets approved: https://github.com/tc39/proposal-weakrefs#finalizers
     */
    cleanup(): void;
    bootstrapStore(): Promise<void>;
    setupInterval(): void;
    setupStoreSubscribe(): void;
    setupLifecycleEvents(): void;
    /**
       Applies the supplied closure to the currently loaded set of plugins.
       NOTE: This does not apply to plugins contained within DestinationPlugins.
  
       - Parameter closure: A closure that takes an plugin to be operated on as a parameter.
  
    */
    apply(closure: (plugin: Plugin) => void): void;
    /**
       Adds a new plugin to the currently loaded set.
  
       - Parameter plugin: The plugin to be added.
       - Returns: Returns the name of the supplied plugin.
  
    */
    add({ plugin }: {
        plugin: Plugin;
    }): void;
    private addPlugin;
    private isNonSegmentDestinationPlugin;
    /**
       Removes and unloads plugins with a matching name from the system.
  
       - Parameter pluginName: An plugin name.
    */
    remove({ plugin }: {
        plugin: Plugin;
    }): void;
    process(incomingEvent: SegmentEvent): void;
    trackDeepLinks(): Promise<void>;
    onUpdateStore(): void;
    flushRetry(): Promise<void>;
    private tick;
    flush(): Promise<void>;
    screen(name: string, options?: JsonMap): void;
    track(eventName: string, options?: JsonMap): void;
    identify(userId: string, userTraits?: UserTraits): void;
    group(groupId: string, groupTraits?: GroupTraits): void;
    alias(newUserId: string): void;
    /**
     * Called once when the client is first created
     *
     * Detect and save the the currently installed application version
     * Send application lifecycle events if trackAppLifecycleEvents is enabled
     *
     * Exactly one of these events will be sent, depending on the current and previous version:s
     * Application Installed - no information on the previous version, so it's a fresh install
     * Application Updated - the previous detected version is different from the current version
     * Application Opened - the previously detected version is same as the current version
     */
    checkInstalledVersion(): Promise<void>;
    /**
     * AppState event listener. Called whenever the app state changes.
     *
     * Send application lifecycle events if trackAppLifecycleEvents is enabled.
     *
     * Application Opened - only when the app state changes from 'inactive' or 'background' to 'active'
     *   The initial event from 'unknown' to 'active' is handled on launch in checkInstalledVersion
     * Application Backgrounded - when the app state changes from 'inactive' or 'background' to 'active
     *
     * @param nextAppState 'active', 'inactive', 'background' or 'unknown'
     */
    handleAppStateChange(nextAppState: AppStateStatus): void;
    reset(): void;
}
